@@ -10,6 +10,7 @@
     SPDX-FileCopyrightText: 2002 Joseph Wenninger <jowenn@kde.org>
     SPDX-FileCopyrightText: 2003 Andras Mantia <amantia@kde.org>
     SPDX-FileCopyrightText: 2005-2006 Hamish Rodda <rodda@kde.org>
+    SPDX-FileCopyrightText: 2023 Kai Uwe Broulik <kde@broulik.de>
 
     SPDX-License-Identifier: LGPL-2.0-only
 */
@@ -19,43 +20,47 @@
 #include <QMenu>
 #include <QToolBar>
 
+#include <memory>
+
 class KToolBarPopupActionPrivate
 {
 public:
-    KToolBarPopupActionPrivate()
-        : m_popupMode(QToolButton::MenuButtonPopup)
-#if KWIDGETSADDONS_BUILD_DEPRECATED_SINCE(5, 78)
-        , stickyMenu(true)
-#endif
+    KToolBarPopupActionPrivate(KToolBarPopupAction *q)
+        : q(q)
+        , m_popupMode(KToolBarPopupAction::MenuButtonPopup)
+        , m_popupMenu(new QMenu())
     {
     }
 
-    QToolButton::ToolButtonPopupMode m_popupMode;
-#if KWIDGETSADDONS_BUILD_DEPRECATED_SINCE(5, 78)
-    bool stickyMenu : 1;
-#endif
+    void applyPopupMode(QToolButton *button, KToolBarPopupAction::PopupMode mode)
+    {
+        if (mode == KToolBarPopupAction::NoPopup) {
+            button->setMenu(nullptr);
+            button->setPopupMode(QToolButton::InstantPopup);
+        } else {
+            // In case the menu is explicitly overridden, use it.
+            // QAction::menu() is automatically applied thanks to QToolButton::setDefaultAction().
+            if (!q->menu()) {
+                button->setMenu(m_popupMenu.get());
+            }
+            button->setPopupMode(static_cast<QToolButton::ToolButtonPopupMode>(mode));
+        }
+    }
+
+    KToolBarPopupAction *q;
+    KToolBarPopupAction::PopupMode m_popupMode;
+    std::unique_ptr<QMenu> m_popupMenu;
 };
 
 KToolBarPopupAction::KToolBarPopupAction(const QIcon &icon, const QString &text, QObject *parent)
     : QWidgetAction(parent)
-    , d(new KToolBarPopupActionPrivate)
+    , d(new KToolBarPopupActionPrivate(this))
 {
     setIcon(icon);
     setText(text);
-    setMenu(new QMenu);
 }
 
-KToolBarPopupAction::~KToolBarPopupAction()
-{
-    delete menu();
-}
-
-#if KWIDGETSADDONS_BUILD_DEPRECATED_SINCE(5, 0)
-QMenu *KToolBarPopupAction::popupMenu() const
-{
-    return menu();
-}
-#endif
+KToolBarPopupAction::~KToolBarPopupAction() = default;
 
 QWidget *KToolBarPopupAction::createWidget(QWidget *_parent)
 {
@@ -69,7 +74,7 @@ QWidget *KToolBarPopupAction::createWidget(QWidget *_parent)
     button->setIconSize(parent->iconSize());
     button->setToolButtonStyle(parent->toolButtonStyle());
     button->setDefaultAction(this);
-    button->setPopupMode(d->m_popupMode);
+    d->applyPopupMode(button, d->m_popupMode);
 
     connect(parent, &QToolBar::iconSizeChanged, button, &QAbstractButton::setIconSize);
     connect(parent, &QToolBar::toolButtonStyleChanged, button, &QToolButton::setToolButtonStyle);
@@ -78,69 +83,26 @@ QWidget *KToolBarPopupAction::createWidget(QWidget *_parent)
     return button;
 }
 
-#if KWIDGETSADDONS_BUILD_DEPRECATED_SINCE(5, 78)
-bool KToolBarPopupAction::delayed() const
-{
-    return popupMode() != QToolButton::InstantPopup;
-}
-#endif
-
-#if KWIDGETSADDONS_BUILD_DEPRECATED_SINCE(5, 78)
-void KToolBarPopupAction::setDelayed(bool delayed)
-{
-    if (delayed) {
-        if (d->stickyMenu) {
-            setPopupMode(QToolButton::MenuButtonPopup);
-        } else {
-            setPopupMode(QToolButton::DelayedPopup);
-        }
-    } else {
-        setPopupMode(QToolButton::InstantPopup);
-    }
-}
-#endif
-
-#if KWIDGETSADDONS_BUILD_DEPRECATED_SINCE(5, 78)
-bool KToolBarPopupAction::stickyMenu() const
-{
-    if (popupMode() == QToolButton::InstantPopup) {
-        return d->stickyMenu;
-    } else if (popupMode() == QToolButton::MenuButtonPopup) {
-        return true;
-    } else { // DelayedPopup
-        return false;
-    }
-}
-#endif
-
-#if KWIDGETSADDONS_BUILD_DEPRECATED_SINCE(5, 78)
-void KToolBarPopupAction::setStickyMenu(bool sticky)
-{
-    if (popupMode() != QToolButton::InstantPopup) {
-        if (sticky) {
-            setPopupMode(QToolButton::MenuButtonPopup);
-        } else {
-            setPopupMode(QToolButton::DelayedPopup);
-        }
-    }
-
-    d->stickyMenu = sticky;
-}
-#endif
-
-QToolButton::ToolButtonPopupMode KToolBarPopupAction::popupMode() const
+KToolBarPopupAction::PopupMode KToolBarPopupAction::popupMode() const
 {
     return d->m_popupMode;
 }
 
-void KToolBarPopupAction::setPopupMode(QToolButton::ToolButtonPopupMode popupMode)
+void KToolBarPopupAction::setPopupMode(KToolBarPopupAction::PopupMode popupMode)
 {
     d->m_popupMode = popupMode;
 
     const auto widgets = createdWidgets();
     for (QWidget *widget : widgets) {
         if (auto *button = qobject_cast<QToolButton *>(widget)) {
-            button->setPopupMode(d->m_popupMode);
+            d->applyPopupMode(button, popupMode);
         }
     }
 }
+
+QMenu *KToolBarPopupAction::popupMenu() const
+{
+    return d->m_popupMenu.get();
+}
+
+#include "moc_ktoolbarpopupaction.cpp"
